import csv
import numpy as np
import matplotlib.pyplot as plt
from os import path

class Plotting:
    def __init__(self, len_x, len_y, N_Val_Samples, x_s, x_e, y_s, y_e):
        self.lenX = len_x
        self.lenY = len_y  # We set it rectangular
        self.v_size = N_Val_Samples # 10
        self.rows_in = (self.lenY - 2)
        self.cols_in = (self.lenX - 2)
        self.NGridPts_int = self.rows_in * self.cols_in
        self.NGridPts_Bdry = self.lenX * self.lenY - self.NGridPts_int
        self.X = None
        self.Y = None
        self.Numerical_Sol = None
        self.ResNet_Prediction = None
        self.Boundary = None
        self.x_s = x_s
        self.x_e = x_e
        self.y_s = y_s
        self.y_e = y_e

        # Create and store the grid: Specify domain and grid size
    def create_grid(self):
        x = np.linspace(self.x_s, self.x_e, self.lenX)
        y = np.linspace(self.y_s, self.y_e, self.lenY)
        self.X, self.Y = np.meshgrid(x, y)

    def initialize(self):
        t = self.lenX * self.lenY
        ns = np.zeros((self.v_size, t), dtype=float)
        self.Numerical_Sol = np.reshape(ns, (self.v_size, self.lenX, self.lenY))
        self.ResNet_Prediction = np.reshape(np.zeros((self.v_size, self.lenX * self.lenY)), (self.v_size, self.lenX, self.lenY))
        self.Boundary = np.zeros((self.v_size, self.NGridPts_Bdry))

    def set_soln_boundary(self):
        bottom = self.Boundary[::, 0:self.lenX:1]
        r_s = self.lenX + self.lenY - 2
        left = self.Boundary[::, self.lenX:r_s:1]
        t_s = r_s + self.lenY - 2
        right = self.Boundary[::, r_s:t_s:1]
        top = self.Boundary[::, t_s:t_s+self.lenX:1]
        self.Numerical_Sol[::, 0, ::] = self.ResNet_Prediction[::, 0, ::] = bottom
        self.Numerical_Sol[::, 1:self.lenY-1:1, 0] = self.ResNet_Prediction[::, 1:self.lenY-1:1, 0] = left
        self.Numerical_Sol[::, 1:self.lenY-1:1, self.lenX-1] = self.ResNet_Prediction[::, 1:self.lenY-1:1, self.lenX-1] = right
        self.Numerical_Sol[::, self.lenY - 1, ::] = self.ResNet_Prediction[::, self.lenY - 1, ::] = top

    def read_data(self):
        with open('output_validation.dat', 'r') as f1:
            self.Numerical_Sol[::, 1:self.lenY - 1:1, 1:self.lenX - 1:1] = np.reshape([x.strip().split(' ') for x in f1],
                                                                           (self.v_size, self.rows_in, self.cols_in))
        with open('valid_out.dat', 'r') as f2:
            self.ResNet_Prediction[::, 1:self.lenY - 1:1, 1:self.lenX - 1:1] = np.reshape([x.strip().split(' ') for x in f2],
                                                                               (self.v_size, self.rows_in, self.cols_in))
        with open('features_validation.dat', 'r') as f3:
            bdry_plus_f = np.reshape([x.strip().split(' ') for x in f3], (self.v_size, (self.NGridPts_Bdry + self.NGridPts_int)))
            self.Boundary = bdry_plus_f[:, 0:self.NGridPts_Bdry:1]

        with open('loss_history_pe.dat', 'r') as f5:
            self.loss = np.loadtxt('loss_history_pe.dat')
            self.iter = self.loss[:, 0]
            self.training_loss = self.loss[:, 1]
            self.val_loss = self.loss[:, 2]
            self.MAPE_train = self.loss[:, 3]
            self.MAPE_val = self.loss[:, 4]


    def plot_and_save_figures(self, Nplots):
        # Number of figs to be generated! Entire validation set is too much!
        colorinterpolation = 100
        colourMap = plt.cm.jet
        for v in range(Nplots):
            fig = plt.figure()
            fig.suptitle('Comparison of temperature contours', fontsize= 14)
            plt.title("Ground truth solution- " + str(v))
            plt.subplot(1, 2, 1)
            plt.title("Numerical Solution Data- " + str(v))
            plt.contourf(self.X, self.Y, self.Numerical_Sol[v], colorinterpolation, cmap=colourMap)
            plt.subplot(1, 2, 2)
            plt.title("ResNet Prediction Data- " + str(v))
            plt.contourf(self.X, self.Y, self.ResNet_Prediction[v], colorinterpolation, cmap=colourMap)
            # TO DO: FIX THE COLOR SCALE FROM -1,1 MAYBE
            cb2 = plt.colorbar()
            cb2.set_label('Temperature')
            plt.show()
            fig.savefig(path.join(outpath, "Data_{0}_ResNet_Pred.png".format(v)))
            cb2.remove()

    def plot_loss(self):
        fig, ax = plt.subplots()  # Create a figure and an axes.
        ax.plot(self.iter, self.training_loss, label='training')
        ax.plot(self.iter, self.val_loss, label='validation')
        plt.yscale('log')
        ax.set_xlabel('Epochs')  # Add an x-label to the axes.
        ax.set_ylabel('Mean squared error')  # Add a y-label to the axes.
        ax.set_title("Loss history")  # Add a title to the axes.
        ax.legend()  # Add a legend.
        plt.savefig(path.join(outpath, "losses.png"))
        plt.show()


    def plot_MAPE(self):
        fig, ax = plt.subplots()  # Create a figure and an axes.
        ax.plot(self.iter, self.MAPE_train, label='training')
        ax.plot(self.iter, self.MAPE_val, label='validation')
        plt.yscale('log')
        ax.set_xlabel('Epochs')  # Add an x-label to the axes.
        ax.set_ylabel('Mean Absolute Percentage Error (MAPE)')  # Add a y-label to the axes.
        ax.set_title("Loss history")  # Add a title to the axes.
        ax.legend()  # Add a legend.
        plt.savefig(path.join(outpath, "MAPE.png"))
        plt.show()



# Input Square Domain Boundaries
x_s = -1.
x_e = 1.
y_s = -1.
y_e = 1.
len_x = 8
len_y = 8
N_Val_Samples = 1000

#Number of plots to be made and saved out of total validation samples
Nplots = 10

#Location where all the images will be stored
outpath = "/home/multigrid/Master Thesis/LayerParallelLearning/plotting_routines/Poisson"


P = Plotting(len_x, len_y, N_Val_Samples, x_s, x_e, y_s, y_e)
P.initialize()
P.create_grid()
P.read_data()
P.set_soln_boundary()
P.plot_and_save_figures(Nplots)
P.plot_loss()
P.plot_MAPE()

'''
fig = plt.figure()
plt.title("Loss Vs number of optimization iterations")
plt.subplot(1, 2, 1)
plt.title("Training loss")
plt.plot(self.loss[::][0], self.loss[::][1] , 'o-', color='g')
plt.subplot(1, 2, 2)
plt.title("Validation Loss")
plt.plot(self.loss[::][0], self.loss[::][2] , 'o-', color='b')
plt.show()
fig.savefig(path.join(outpath, "loss_history.png"))
'''